==> jabber-transport-0.9.tar.gz: jabber-transport-0.9/src/users.c <==
#include "jserver.h"

HASHTABLE js_users = NULL;

int _js_users_del(void *arg, const void *key, void *data)
{
    udata u = (udata)data;
    xdb curd;

    if(u->ref > 0 || u->sessions != NULL)
        return 1;

    fprintf(stderr,"[%s]",u->user);
    log_debug(ZONE,"freeing %s",u->user);

    for(curd = u->x_cache; curd != NULL; curd = curd->next)
        xmlnode_free(curd->data);
    ghash_remove(js_users,u->user);
    ppdb_free(u->p_cache);
    pool_free(u->p);

    return 1;
}

/* thread that flushes old users from memory */
void *js_users_main(void *arg)
{
    if(js_users == NULL)
        js_users = ghash_create((KEYHASHFUNC)str_hash_code,(KEYCOMPAREFUNC)j_strcmp);

    log_debug(ZONE,"THREAD:USERS starting");

    while(1)
    {
        pth_sleep(5);
 
        /* free user struct if we can */
        ghash_walk(js_users,_js_users_del,NULL);
    }
}



/* locates user data record
 * looks in current list,
 * else looks in xdb and creates new entry
 * else returns NULL (not a user)
 */
udata js_user(char *user)
{
    pool p;
    udata cur, newu;
    char *ustr, *u;

    if(user == NULL) return NULL;

    if(js_users == NULL)
        js_users = ghash_create((KEYHASHFUNC)str_hash_code,(KEYCOMPAREFUNC)j_strcmp);

    for(ustr = u = strdup(user); *ustr != '\0'; ustr++)
        *ustr = tolower(*ustr);

    log_debug(ZONE,"js_user(%s)",u);

    cur = ghash_get(js_users,u);
    if(cur != NULL)
    {
        free(u);
        return cur;
    }
 
    log_debug(ZONE,"js_user not current");

    /* create a udata struct */
    p = pool_new();
    newu = pmalloc(p, sizeof(_udata));
    memset(newu, '\0', sizeof(_udata));
    newu->p = p;
    newu->user = pstrdup(p, u);
    free(u);

    if(!js_xdb_get(newu, NS_AUTH))
    { /* no password, no user! */
        pool_free(newu->p);
        return NULL;
    }

    ghash_put(js_users,newu->user,newu);
    return newu;
}

int _js_users_exit(void *arg, const void *key, void *data)
{
    udata u = (udata)data;
    session s;

    for(s = u->sessions; s != NULL; s = s->next)
        js_session_end(s, "Server Exiting");

    return 1;
}

void js_users_exit(void)
{
    udata curu;
    session curs;

    /* tell all the sessions to exit */
    ghash_walk(js_users,_js_users_exit,NULL);
}

==> jabber-transport-1.0pre1.tar.gz: jabber-transport-1.0pre1/src/users.c <==
#include "jserver.h"

HASHTABLE js_users = NULL;

int _js_users_del(void *arg, const void *key, void *data)
{
    udata u = (udata)data;
    xdb curd;

    if(u->ref > 0 || u->sessions != NULL)
        return 1;

    fprintf(stderr,"[%s]",u->user);
    log_debug(ZONE,"freeing %s",u->user);

    for(curd = u->x_cache; curd != NULL; curd = curd->next)
        xmlnode_free(curd->data);
    ghash_remove(js_users,u->user);
    ppdb_free(u->p_cache);
    pool_free(u->p);

    return 1;
}

/* thread that flushes old users from memory */
void *js_users_main(void *arg)
{
    if(js_users == NULL)
        js_users = ghash_create(HASH_PRIME,(KEYHASHFUNC)str_hash_code,(KEYCOMPAREFUNC)j_strcmp);

    log_debug(ZONE,"THREAD:USERS starting");

    while(1)
    {
        pth_sleep(5);
 
        /* free user struct if we can */
        ghash_walk(js_users,_js_users_del,NULL);
    }
}



/* locates user data record
 * looks in current list,
 * else looks in xdb and creates new entry
 * else returns NULL (not a user)
 */
udata js_user(char *user)
{
    pool p;
    udata cur, newu;
    char *ustr, *u;

    if(user == NULL) return NULL;

    if(js_users == NULL)
        js_users = ghash_create(HASH_PRIME,(KEYHASHFUNC)str_hash_code,(KEYCOMPAREFUNC)j_strcmp);

    for(ustr = u = strdup(user); *ustr != '\0'; ustr++)
        *ustr = tolower(*ustr);

    log_debug(ZONE,"js_user(%s)",u);

    cur = ghash_get(js_users,u);
    if(cur != NULL)
    {
        free(u);
        return cur;
    }
 
    log_debug(ZONE,"js_user not current");

    /* create a udata struct */
    p = pool_new();
    newu = pmalloc(p, sizeof(_udata));
    memset(newu, '\0', sizeof(_udata));
    newu->p = p;
    newu->user = pstrdup(p, u);
    free(u);

    if(!js_xdb_get(newu, NS_AUTH))
    { /* no password, no user! */
        pool_free(newu->p);
        return NULL;
    }

    ghash_put(js_users,newu->user,newu);
    return newu;
}

int _js_users_exit(void *arg, const void *key, void *data)
{
    udata u = (udata)data;
    session s;

    for(s = u->sessions; s != NULL; s = s->next)
        js_session_end(s, "Server Exiting");

    return 1;
}

void js_users_exit(void)
{
    udata curu;
    session curs;

    /* tell all the sessions to exit */
    ghash_walk(js_users,_js_users_exit,NULL);
}
